@@ -21,7 +21,7 @@
 
 #define SERVER_UPDATE_MESSAGE "acnuc stop for update\n"
 /* seules fctions utilisables hors de ce fichier pour ecrire sur socket */
-int sock_fputs(raa_db_access *f, char *l);
+int sock_fputs(raa_db_access *f, const char *l);
 int sock_flush(raa_db_access *f);
 
 /* some prototypes */
@@ -103,7 +103,7 @@ return mswin_sock_flush(raa_current_db);
 }
 
 
-int sock_fputs(raa_db_access  *raa_current_db, char *s)
+int sock_fputs(raa_db_access  *raa_current_db, const char *s)
 {
 int l, r;
 
@@ -128,7 +128,7 @@ return 0;
 
 #else
 
-int sock_fputs(raa_db_access  *raa_current_db, char *s)
+int sock_fputs(raa_db_access  *raa_current_db, const char *s)
 {
 if(raa_current_db == NULL) return EOF;
 return fputs(s, raa_current_db->raa_sockfdw);
@@ -260,7 +260,7 @@ badracnuc,  /* 8 enviroment variables racnuc or acnuc undefined or inadequate */
 nosocket /* 9 no socket was opened yet */
 };
 
-int raa_acnucopen (char *clientid, raa_db_access **psock) 
+int raa_acnucopen (const char *clientid, raa_db_access **psock) 
 /* opens the acnuc db using the environment variable racnuc, or, if undefined, acnuc,
 that should be defined to an url of the form
 raa://pbil.univ-lyon1.fr:5558/embl
@@ -281,7 +281,7 @@ return err;
 }
 
 
-int raa_acnucopen_alt (char *serveurName, int port, char *db_name, char *clientid, raa_db_access **p) 
+int raa_acnucopen_alt (const char *serveurName, int port, const char *db_name, const char *clientid, raa_db_access **p) 
 /*
 clientid: NULL or a string identifying the client
 */
@@ -298,7 +298,7 @@ int err;
  }
 
 
-int raa_open_socket(char *serveurName, int port, char *clientid, raa_db_access **psock)
+int raa_open_socket(const char *serveurName, int port, const char *clientid, raa_db_access **psock)
 /*
 clientid: NULL or a string identifying the client
 */
@@ -374,7 +374,7 @@ raa_current_db->raa_sockfdw = fdopen(raa_snum,"a");
 
 extern void raa_MD5String (char *in_string, char out_digest[33]);
 
-int raa_opendb_pw(raa_db_access *raa_current_db, char *db_name, void *ptr, char *(*getpasswordf)(void *) )
+int raa_opendb_pw(raa_db_access *raa_current_db, const char *db_name, void *ptr, char *(*getpasswordf)(void *) )
 /*
 getpasswordf: pointer to function that gets called if a password is needed
 ptr: pointer to data passed to the getpasswordf function
@@ -494,7 +494,7 @@ return 0;
 }
 
 
-int raa_opendb(raa_db_access *raa_current_db, char *db_name)
+int raa_opendb(raa_db_access *raa_current_db, const char *db_name)
 {
 return raa_opendb_pw(raa_current_db, db_name, NULL, NULL);
 }
@@ -537,7 +537,7 @@ return 0;
 static int fill_gfrag_buf(raa_db_access *raa_current_db, int nsub, int first)
 {
 char *p, *line;
-int lu, l, length, wasfull;
+int lu, l, wasfull;
 
 sock_printf(raa_current_db,"gfrag&number=%d&start=%d&length=%d\n", nsub, first, RAA_GFRAG_BSIZE);
 /* retour:  length=xx&...the seq...\n */
@@ -546,7 +546,6 @@ if(line == NULL) return 0;
 if(strncmp(line, "length=", 7) != 0 || (p = strchr(line, '&')) == NULL ) {
 	return 0;
 	}
-length = atoi(line + 7);
 lu = strlen(++p);
 memcpy(raa_current_db->gfrag_data.buffer, p, lu);
 while(! wasfull) {
@@ -2022,7 +2021,7 @@ int raa_nextmatchkey(raa_db_access *raa_current_db, int num, char *pattern, char
 /*  *matching returned in static memory */
 {
 struct raa_matchkey *data;
-int no_more, count;
+int no_more=0, count;
 
 if(raa_current_db == NULL) return 0;
 if(num == 2) {
@@ -2394,7 +2393,7 @@ if(strchr(name, '.') != NULL) { /* subsequence */
 				!strncmp(p,"FT        ",10) );
 	}
 else	{ /* parent sequence */
-	if( (p = raa_read_annots(raa_current_db, pinf, div)) == NULL) return text;
+	if( raa_read_annots(raa_current_db, pinf, div) == NULL) return text;
 	p = raa_next_annots(raa_current_db, NULL);
 	if(raa_current_db->nbrf) {
 		deb=17;
@@ -2673,11 +2672,11 @@ static void ajout_branche(raa_node *pere, raa_node *fils)
 /* adds a pere->fils branch. The last added branch is the first child of pere.
  */
 {
-struct raa_pair *point, *nouveau;
+struct raa_pair *nouveau;
 
 nouveau = (struct raa_pair *)calloc(1,sizeof(struct raa_pair));
 nouveau->value = fils;
-if( (point = pere->list_desc) == NULL) {
+if( pere->list_desc == NULL) {
 	pere->list_desc = nouveau;
 	}
 else	{
