@@ -16,19 +16,16 @@ void refresh_callback(Fl_Widget *ob, void *data);
 void remove_gaps_callback(Fl_Widget *ob, void *data);
 void remove_numbers_callback(Fl_Widget *ob, void *data);
 void to_upcase_callback(Fl_Widget *ob, void *data);
-void load_seq_callback(Fl_Widget *ob, void *data);
+void load_seq_callback(void *data);
 void load_seq_dialog(SEA_VIEW *view);
-int add_seq_to_align(SEA_VIEW *view, char *newname, char *newseq, 
-	int lenseq);
 char complement_base(char old);
 void edit_comments_dialog(SEA_VIEW *view);
 void edit_sequence_dialog(SEA_VIEW *view);
-void close_editseqwin_callback(Fl_Widget *ob, void *data);
 int load_sequence_for_edit(struct editseq *editdata);
 char *renumber_seq(char *seq);
 void renumber_callback(Fl_Widget *ob, void *data);
-void edit_sequence_callback(Fl_Widget *ob, void *data);
-void update_comments_callback(Fl_Widget *ob, void *data);
+void edit_sequence_callback(struct editseq *editdata);
+void update_comments_callback(void *data);
 int load_comments(SEA_VIEW *view, Fl_Input *input, Fl_Widget *name);
 char *cre_consensus(SEA_VIEW *view, char *newname);
 Fl_Widget *cre_adjusted_button(int x, int y, int *w, int h, 
@@ -38,9 +35,10 @@ Fl_Widget *cre_adjusted_button(int x, int y, int *w, int h,
 /* external prototypes */
 extern void hide_window_callback(Fl_Widget *ob, void *data);
 extern void allonge_seqs(char **seq, int totseqs, int maxlen, int *eachlength,
-	int tot_comment_lines, char **comment_line);
+	int tot_comment_lines, char **comment_line, char **pregion_line);
 extern int compute_wid_names(SEA_VIEW *view, int totseqs);
 extern int compute_size_params(SEA_VIEW *view, int force_recompute);
+extern int calc_max_seq_length(int seq_length, int tot_seqs);
 
 	
 void clear_callback(Fl_Widget *ob, void *data)
@@ -117,7 +115,7 @@ seq_input->value(new_seq);
 }
 
 
-void load_seq_callback(Fl_Widget *ob, void *data)
+void load_seq_callback(void *data)
 {
 Fl_Input *seq_name_input, *seq_input;
 SEA_VIEW *view;
@@ -140,7 +138,6 @@ if(newseq == NULL) {
 	fl_alert("Not enough memory");
 	return;
 	}
-ob->window()->hide();
 p = tmp; q = newseq; lenseq = 0;
 while(*p != 0) {
 	if(*p != ' ' && *p != '\n' && *p != '\t') {
@@ -173,53 +170,46 @@ return (Fl_Widget *)obj;
 
 void load_seq_dialog(SEA_VIEW *view)
 {
-static Fl_Window *load_form;
-static int first = TRUE;
-static Fl_Input *seq_input, *seq_name_input;
-static view_name_struct view_name;
+  Fl_Window *load_form;
+  Fl_Input *seq_input, *seq_name_input;
+  view_name_struct view_name;
+  Fl_Widget *cancel, *add, *obj;
+  int fin, width;
+  load_form = new Fl_Window(490,530);
+  load_form->box(FL_FLAT_BOX);
+  load_form->label("Sequence Loading");
+  
+  int curr_y = 5;
+  fin = 5;
+  Fl_Group *top = new Fl_Group(fin,curr_y,load_form->w() - 2 * fin, 25);
+  static char label[] = "Seq. name:";
+  fl_font(FL_HELVETICA, FL_NORMAL_SIZE);
+  width = (int)fl_width(label) + 4 * 2;
+  Fl_Box *name_box = new Fl_Box(FL_UP_BOX, 
+	  fin, top->y(), width, top->h(), label);
+  name_box->labelsize(FL_NORMAL_SIZE);
+  name_box->box(FL_NO_BOX);
+  fin += width + 5;
 
-if(first) {
-	Fl_Widget *obj;
-	int fin, width;
-	first = FALSE;
-	load_form = new Fl_Window(490,530);
-	load_form->box(FL_FLAT_BOX);
-	load_form->label("Sequence Loading");
-	
-	int curr_y = 5;
-	fin = 5;
-	Fl_Group *top = new Fl_Group(fin,curr_y,load_form->w() - 2 * fin, 25);
-	static char label[] = "Seq. name:";
-	fl_font(FL_HELVETICA, FL_NORMAL_SIZE);
-	width = (int)fl_width(label) + 4 * 2;
-	Fl_Box *name_box = new Fl_Box(FL_UP_BOX, 
-		fin, top->y(), width, top->h(), label);
-	name_box->labelsize(FL_NORMAL_SIZE);
-	name_box->box(FL_NO_BOX);
-	fin += width + 5;
-
-	seq_name_input = new Fl_Input(fin,top->y(),100,top->h(),"");
-	seq_name_input->type(FL_NORMAL_INPUT);
-	fin += seq_name_input->w() + 5;
-	
-	obj = cre_adjusted_button(fin,top->y(),&width,top->h(),
-		"Add to alignment");
-	obj->callback(load_seq_callback, &view_name);
-	fin += width + 5;
-	
-	obj = cre_adjusted_button(fin,top->y(),&width,top->h(),
-		"Cancel");
-	obj->callback(hide_window_callback, NULL);
-	((Fl_Button *)obj)->shortcut(FL_COMMAND | 'w');
-	fin += width;
-	
-	top->resizable(NULL);
-	top->size(fin - top->x(), top->h());
-	top->end();
-	
-	curr_y += top->h() + 15;
-	seq_input = new 
-		Fl_Input(top->x(), curr_y, load_form->w() - 2 * top->x(), 450,
+  seq_name_input = new Fl_Input(fin,top->y(),100,top->h(),"");
+  seq_name_input->type(FL_NORMAL_INPUT);
+  fin += seq_name_input->w() + 5;
+  
+  add = cre_adjusted_button(fin,top->y(),&width,top->h(),
+	  "Add to alignment");
+  fin += width + 5;
+  
+  cancel = cre_adjusted_button(fin,top->y(),&width,top->h(),
+	  "Cancel");
+  ((Fl_Button *)cancel)->shortcut(FL_COMMAND | 'w');
+  fin += width;
+  
+  top->resizable(NULL);
+  top->size(fin - top->x(), top->h());
+  top->end();
+  
+  curr_y += top->h() + 15;
+  seq_input = new Fl_Input(top->x(), curr_y, load_form->w() - 2 * top->x(), 450,
 #if defined(__APPLE__)
 	    ""
 #elif defined(WIN32) 
@@ -228,64 +218,73 @@ if(first) {
 	    "Type or paste sequence with middle mouse button in panel below"
 #endif
 	    );
-	seq_input->type(FL_MULTILINE_INPUT);
-	seq_input->align(FL_ALIGN_TOP);
-	seq_input->textfont(FL_COURIER);
-	seq_input->textsize(12);
-	fin = top->x();
-	curr_y += seq_input->h() + 5;
-	Fl_Group *bottom = new Fl_Group(fin, curr_y, top->w(), top->h() );
-	obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(), "Refresh");
-	obj->callback(refresh_callback, &view_name);
-	fin += width + 5;
-	
-	obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(), "Clear");
-	obj->callback(clear_callback, &view_name);
-	fin += width + 5;
-	
-	obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(),
-		"Remove gaps");
-	obj->callback(remove_gaps_callback, seq_input);
-	fin += width + 5;
-	
-	obj = cre_adjusted_button(fin,curr_y,&width,bottom->h(),
-		"Remove numbers");
-	obj->callback(remove_numbers_callback, seq_input);
-	fin += width + 5;
-	
-	obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(),
-		"to UPPER");
-	obj->callback(to_upcase_callback, seq_input);
-	fin += width;
-	
-	bottom->resizable(NULL);
-	bottom->size(fin - bottom->x(), bottom->h());
-	bottom->end();
-	
-	load_form->resizable(seq_input);
-	
-	view_name.seq_name_field = seq_name_input;
-	view_name.seq_field = seq_input;
-	load_form->end();
-        load_form->position((Fl::w() - load_form->w())/2, 
-		(Fl::h() - load_form->h())/2);
-	}
-seq_name_input->value("");
-seq_input->value("");
-view_name.view = view;
+  seq_input->type(FL_MULTILINE_INPUT);
+  seq_input->align(FL_ALIGN_TOP);
+  seq_input->textfont(FL_COURIER);
+  seq_input->textsize(12);
+  fin = top->x();
+  curr_y += seq_input->h() + 5;
+  Fl_Group *bottom = new Fl_Group(fin, curr_y, top->w(), top->h() );
+  obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(), "Refresh");
+  obj->callback(refresh_callback, &view_name);
+  fin += width + 5;
+
+  obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(), "Clear");
+  obj->callback(clear_callback, &view_name);
+  fin += width + 5;
+
+  obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(),
+	"Remove gaps");
+  obj->callback(remove_gaps_callback, seq_input);
+  fin += width + 5;
+
+  obj = cre_adjusted_button(fin,curr_y,&width,bottom->h(),
+	"Remove numbers");
+  obj->callback(remove_numbers_callback, seq_input);
+  fin += width + 5;
+
+  obj = cre_adjusted_button(fin, curr_y, &width, bottom->h(),
+	"to UPPER");
+  obj->callback(to_upcase_callback, seq_input);
+  fin += width;
+
+  bottom->resizable(NULL);
+  bottom->size(fin - bottom->x(), bottom->h());
+  bottom->end();
+
+  load_form->resizable(seq_input);
+
+  view_name.seq_name_field = seq_name_input;
+  view_name.seq_field = seq_input;
+  load_form->end();
+  load_form->position((Fl::w() - load_form->w())/2, (Fl::h() - load_form->h())/2);
+  seq_name_input->value("");
+  seq_input->value("");
+  view_name.view = view;
 #if defined(__APPLE__)
 	seq_input->value("\n\n\n\n\n\nType sequence or paste it with cmd-V\n\n\n");
 	seq_input->position(seq_input->size(), 0);
 #endif	
-seq_input->take_focus();
-load_form->show();
+  seq_input->take_focus();
+  load_form->show();
 #ifndef MICRO
-load_form->hotspot(load_form);
+  load_form->hotspot(load_form);
 #endif
+  while (TRUE) {
+    obj = Fl::readqueue();
+    if (!obj) Fl::wait();
+    else if(obj == add || obj == cancel || obj == load_form) {
+      if (obj == add) {
+	load_seq_callback(&view_name);
+      }
+      delete load_form;
+      break;
+    }
+  }
 }
 
 
-int add_seq_to_align(SEA_VIEW *view, char *newname, char *newseq, int lenseq)
+int add_seq_to_align(SEA_VIEW *view, char *newname, char *newseq, int lenseq, bool no_gui)
 /* returns 0 if failed or the rank of newly added sequence
 */
 {
@@ -313,16 +312,17 @@ if(new_n != NULL) view->seqname = new_n;
 
 if(new_s == NULL || new_n == NULL) goto nomem;
 if(lenseq > view->max_seq_length) {
-  view->max_seq_length = FL_max(lenseq * 1.3, lenseq + 500);
+  view->max_seq_length = calc_max_seq_length(lenseq, view->tot_seqs);
   allonge_seqs(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
-	       view->tot_comment_lines, view->comment_line);
-  view->region_line = (char*)realloc(view->region_line, view->max_seq_length + 1);
-  for (int i = 0; i < view->tot_seqs; i++) free(view->col_rank[i]);
-  free(view->col_rank);
-  view->col_rank = prepcolranks(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
-	       ( view->protein ? get_color_for_aa : get_color_for_base ), 
-	       view->numb_gc, view->allow_lower);
-	}
+	       view->tot_comment_lines, view->comment_line, &view->region_line);
+  if (!no_gui) {
+    for (int i = 0; i < view->tot_seqs; i++) free(view->col_rank[i]);
+    free(view->col_rank);
+    view->col_rank = prepcolranks(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+		 ( view->protein ? get_color_for_aa : get_color_for_base ), 
+		 view->numb_gc, view->allow_lower);
+    }
+  }
 if( (view->seqname[num-1] = (char *)malloc(strlen(newname)+1)) == NULL) 
 	goto nomem;
 strcpy(view->seqname[num-1], newname);
@@ -332,12 +332,12 @@ if(view->comments != NULL) {
 if( (view->sequence[num-1] = (char *)malloc(view->max_seq_length + 1)) == NULL) 
 	goto nomem;
 memcpy(view->sequence[num-1], newseq, lenseq+1);
-if(view->masename == NULL) {
+if(view->masename == NULL && !no_gui) {
 	newname = (char *) fl_input("Name of the new alignment?", "");
 	if(newname == NULL) newname = defname;
 	}
 else newname = view->masename;
-if(view->tot_seqs == 0) {
+if (view->tot_seqs == 0 && !no_gui) {
 	init_dna_scroller(view, 1, newname, is_a_protein_seq(newseq), NULL);
 	set_seaview_modified(view, TRUE);
 	view->DNA_obj->redraw();
@@ -349,7 +349,7 @@ memcpy(newsel, view->each_length, (num-1)*sizeof(int) );
 free(view->each_length);
 view->each_length = newsel;
 view->each_length[num-1] = lenseq;
-if(lenseq > view->seq_length) {
+if (lenseq > view->seq_length && !no_gui) {
 	double x; int l;
 	view->seq_length = lenseq;
 	x = ( (double) view->tot_sites ) / ( view->seq_length + 3 ); 
@@ -359,7 +359,7 @@ if(lenseq > view->seq_length) {
 	if(l<1) l=1;
 	((Fl_Slider*)view->horsli)->bounds(1,l);
 	}
-if(view->numb_dnacolors > 1) {
+if(view->numb_dnacolors > 1 && !no_gui) {
 	if( (newcol = (char **)malloc(num*sizeof(char *))) == NULL) 
 		goto nomem;
 	memcpy(newcol, view->col_rank, (num-1)*sizeof(char *) );
@@ -390,7 +390,8 @@ view->tot_seqs = num;
 view->cursor_seq = num;
 view->cursor_site = 1;
 view->first_site = 1;
-if(strlen(view->seqname[num - 1]) > view->wid_names) {
+  if (no_gui) return num;
+if(strlen(view->seqname[num - 1]) > (unsigned)view->wid_names) {
   view->wid_names = compute_wid_names(view, num);
   compute_size_params(view, TRUE);
 }
@@ -404,8 +405,8 @@ view->vertsli->redraw();
 view->horsli->redraw();
 return num;
 nomem:
-fl_alert("Not enough memory\nto create the new sequence %s",
-		newname);
+  void (*alert)(const char *,...) = no_gui ? Fl::fatal : fl_alert;
+  alert("Not enough memory\nto create the new sequence %s", newname);
 return 0;
 }
 
@@ -424,67 +425,75 @@ else
 
 void edit_comments_dialog(SEA_VIEW *view)
 {
-static Fl_Window *comments_form;
-static int first = TRUE;
-static Fl_Input *comments_input;
-static Fl_Widget *comments_name;
-static view_name_struct comments_data;
+  Fl_Window *comments_form;
+  Fl_Input *comments_input;
+  Fl_Widget *comments_name;
+  view_name_struct comments_data;
+  Fl_Widget *o, *cancel, *apply;
 
-if(first) {
-	Fl_Widget *obj;
-	int fin, width;
-	first = FALSE;
-	comments_form = new Fl_Window(490,530);
-	comments_form->label("Comments Editing");
-	comments_form->box(FL_FLAT_BOX);
-	
-	Fl_Group *top_group = new Fl_Group(5,5,comments_form->w() - 10, 25);
-	fin = 5;
-	obj = cre_adjusted_button(fin,5,&width,25, "Apply");
-	obj->callback(update_comments_callback, &comments_data);
-	fin += width + 5;
-	
-	obj = cre_adjusted_button(fin,5,&width,25,"Cancel");
-	obj->callback(hide_window_callback, NULL);
-	((Fl_Button *)obj)->shortcut(FL_COMMAND | 'w');
-	fin += width + 5;
-	
-	obj = cre_adjusted_button(fin,5,&width,25,"Refresh");
-	obj->callback(refresh_callback, &comments_data);
-	fin += width + 5;
+  Fl_Widget *obj;
+  int fin, width;
+  comments_form = new Fl_Window(490,530);
+  comments_form->label("Comments Editing");
+  comments_form->box(FL_FLAT_BOX);
+  
+  Fl_Group *top_group = new Fl_Group(5,5,comments_form->w() - 10, 25);
+  fin = 5;
+  apply = cre_adjusted_button(fin,5,&width,25, "Apply");
+  fin += width + 5;
+  
+  cancel = cre_adjusted_button(fin,5,&width,25,"Cancel");
+  ((Fl_Button *)cancel)->shortcut(FL_COMMAND | 'w');
+  fin += width + 5;
+  
+  obj = cre_adjusted_button(fin,5,&width,25,"Refresh");
+  obj->callback(refresh_callback, &comments_data);
+  fin += width + 5;
 
-	comments_name = new Fl_Box(FL_DOWN_BOX, fin, 5, 
-		top_group->x() +top_group->w() - fin, 25, "");
-	comments_name->align(FL_ALIGN_LEFT | FL_ALIGN_CLIP | FL_ALIGN_INSIDE);
-	comments_name->labelfont(FL_COURIER);
-	comments_name->labelsize(FL_NORMAL_SIZE);
+  comments_name = new Fl_Box(FL_DOWN_BOX, fin, 5, 
+	  top_group->x() +top_group->w() - fin, 25, "");
+  comments_name->align(FL_ALIGN_LEFT | FL_ALIGN_CLIP | FL_ALIGN_INSIDE);
+  comments_name->labelfont(FL_COURIER);
+  comments_name->labelsize(FL_NORMAL_SIZE);
 
-	top_group->resizable(comments_name);
-	top_group->end();
-	
-	comments_input = new Fl_Input(5,35,top_group->w(),
-			comments_form->h() - 5 - 35, "");
-	comments_input->type(FL_MULTILINE_INPUT);
-	comments_input->textfont(FL_COURIER);
-	comments_input->textsize(12);
-	comments_data.seq_field = comments_input;
-	comments_form->resizable(comments_input);
-	comments_form->end();
-	comments_form->position( (Fl::w() - comments_form->w())/2, 
-    		(Fl::h() - comments_form->h())/2 );
-	}
-comments_data.view = view;
-if(load_comments(view, comments_input, comments_name)) {
+  top_group->resizable(comments_name);
+  top_group->end();
+  
+  comments_input = new Fl_Input(5,35,top_group->w(),
+		  comments_form->h() - 5 - 35, "");
+  comments_input->type(FL_MULTILINE_INPUT);
+  comments_input->textfont(FL_COURIER);
+  comments_input->textsize(12);
+  comments_data.seq_field = comments_input;
+  comments_form->resizable(comments_input);
+  comments_form->end();
+  comments_form->position( (Fl::w() - comments_form->w())/2, 
+	  (Fl::h() - comments_form->h())/2 );
+  comments_data.view = view;
+  if(load_comments(view, comments_input, comments_name)) {
+	delete comments_form;
 	fl_alert("Not enough memory");
 	return;
 	}
-comments_input->take_focus();
-comments_form->show();
+  comments_input->take_focus();
+  comments_form->show();
 #ifndef MICRO
 	comments_form->hotspot(comments_form);
 #endif
+  while (TRUE) {
+    o = Fl::readqueue();
+    if (!o) Fl::wait();
+    else if (o == apply || o == cancel || o == comments_form) {
+      if (o == apply) {
+	update_comments_callback(&comments_data);
+	}
+      delete comments_form;
+      break;
+      }
+    }
 }
 
+
 struct editseq {
 	Fl_Multiline_Input *input;
 	Fl_Widget *name;
@@ -498,7 +507,7 @@ void edit_sequence_dialog(SEA_VIEW *view)
 struct editseq *editdata;
 int fin, width, num;
 Fl_Window *form;
-Fl_Widget *obj;
+Fl_Widget *obj, *apply, *cancel;
 
 for(num = 0; num <view->tot_seqs; num++) { if(view->sel_seqs[num]) break; }
 if(num >= view->tot_seqs) return;
@@ -509,21 +518,19 @@ editdata->view = view;
 form = new Fl_Window(770,530);
 form->label("Sequence Editing");
 form->box(FL_FLAT_BOX);
-form->callback(close_editseqwin_callback, editdata);
+form->user_data(editdata);
 
 Fl_Group *top_group = new Fl_Group(5,5,form->w() - 10, 25);
 fin = 5;
-obj = cre_adjusted_button(fin,5,&width,25, "Apply");
-obj->callback(edit_sequence_callback, 0);
+apply = cre_adjusted_button(fin,5,&width,25, "Apply");
 fin += width + 5;
 
 obj = cre_adjusted_button(fin,5,&width,25,"Renumber");
 obj->callback(renumber_callback, 0);
 fin += width + 5;
 
-obj = cre_adjusted_button(fin,5,&width,25,"Cancel");
-obj->callback(close_editseqwin_callback, editdata);
-((Fl_Button *)obj)->shortcut(FL_COMMAND | 'w');
+cancel = cre_adjusted_button(fin,5,&width,25,"Cancel");
+((Fl_Button *)cancel)->shortcut(FL_COMMAND | 'w');
 fin += width + 5;
 
 editdata->name = new Fl_Box(FL_DOWN_BOX, fin, 5, 
@@ -546,6 +553,8 @@ form->position( (Fl::w() - form->w())/2,
 		(Fl::h() - form->h())/2 );
 	
 if(load_sequence_for_edit(editdata)) {
+  free(editdata);
+	delete form;
 	fl_alert("Not enough memory");
 	return;
 	}
@@ -554,16 +563,18 @@ form->show();
 form->hotspot(form);
 #endif
 editdata->input->take_focus();
-}
-
-
-void close_editseqwin_callback(Fl_Widget *ob, void *data)
-{
-	free(data);
-	Fl_Window *w = ob->window();
-	if(w == NULL) w = (Fl_Window *)ob;
-	w->hide();
-	Fl::delete_widget( w );
+  while (TRUE) {
+    obj = Fl::readqueue();
+    if (!obj) Fl::wait();
+    else if(obj == apply || obj == cancel || obj == form) {
+      if (obj == apply) {
+	edit_sequence_callback(editdata);
+      }
+      free(editdata);
+      delete form;
+      break;
+    }
+  }
 }
 
 
@@ -625,9 +636,8 @@ free(temp);
 }
 
 
-void edit_sequence_callback(Fl_Widget *ob, void *data)
+void edit_sequence_callback(struct editseq *editdata)
 {
-struct editseq *editdata = (struct editseq *)ob->window()->user_data();
 char *p, *newseq, *q, **newcolseq;
 SEA_VIEW *view;
 int num, l;
@@ -647,10 +657,9 @@ while(*(++p) !=  0) {
 *q = 0;
 l = strlen(newseq);
   if (l > view->max_seq_length) {
-    view->max_seq_length = FL_max(l * 1.3, l + 500);
+    view->max_seq_length = calc_max_seq_length(l, view->tot_seqs);
     allonge_seqs(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
-		 view->tot_comment_lines, view->comment_line);
-    view->region_line = (char*)realloc(view->region_line, view->max_seq_length + 1);
+		 view->tot_comment_lines, view->comment_line, &view->region_line);
     for (int i = 0; i < view->tot_seqs; i++) free(view->col_rank[i]);
     free(view->col_rank);
     view->col_rank = prepcolranks(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
@@ -682,18 +691,16 @@ if(l > view->seq_length) {
 	}
 else view->mod_seq = num;
 view->DNA_obj->redraw();
-ob->window()->do_callback();
 }
 
 
-void update_comments_callback(Fl_Widget *ob, void *data)
+void update_comments_callback(void *data)
 {
 Fl_Input *comments_input;
 SEA_VIEW *view;
 int num, l, num_l;
 char *temp, *p, *q, *r;
 
-ob->window()->hide();
 view = ((view_name_struct *)data)->view;
 if(view->comments == NULL) {
 	view->comments = (char **)calloc(view->tot_seqs , sizeof(char *));
