@@ -1,17 +1,5 @@
 #ifdef __APPLE__
 #import <Cocoa/Cocoa.h>
-#ifdef __ppc__
-#include <QuickTime/QuickTimeComponents.h>
-#endif
-#ifndef NSINTEGER_DEFINED //appears with 10.5 in NSObjCRuntime.h
-#if defined(__LP64__) && __LP64__
-typedef long NSInteger;
-typedef unsigned long NSUInteger;
-#else
-typedef int NSInteger;
-typedef unsigned int NSUInteger;
-#endif
-#endif
 #include "seaview.h"
 #include "pdf_or_ps.h"
 #include <FL/x.H>
