@@ -133,7 +133,7 @@ typedef enum { ALIGN_ALL, ALIGN_SELECTED_SEQS, ALIGN_SITES, PROFILE, UNALIGN, AL
 typedef enum {SHOW_HIDE_FOOTERS, CREATE_FOOTER, DELETE_FOOTER }
 	footers_menu_choice;
 	
-typedef enum {PDF_COLOR, PDF_BW, TEXT_ONLY} pdf_kind;
+typedef enum {PDF_COLOR, PDF_BW, TEXT_ONLY, SVG} pdf_kind;
 
 class vlength_menu // an FL_SUBMENU_POINTER-attached variable-length series of Fl_Menu_Item's
 {
