@@ -28,6 +28,7 @@ struct mon_point {
 };
 struct trait {
 	double xd,yd,xf,yf;
+	int width;
 };
 
 enum operations {show_tree = 1, depl_racine, permutation, subtree, selection, prune_clade, prune_clade_found};
@@ -51,7 +52,8 @@ struct FD_nj_plot {
 	int lost_br_lengths;
 	int has_internal;
 	int root_num;
-	int rooted;
+	int rooted; // true if tree is currently rooted
+	int input_was_rooted; // true if tree read from Newick string was rooted
 	struct noeud **tabtax;
 	struct noeud *racine;
 	char **labels;
@@ -106,5 +108,8 @@ struct FD_nj_plot {
 	Fl_Counter *zoom;
 	Fl_Choice *root_unroot;
 	int page_count;
+  bool branch_width_as_support;
+  float support_threshold_high;
+  float support_threshold_low;
 };
 #endif // TREEDRAW_H
