@@ -9,6 +9,7 @@
 #include <FL/Fl_Return_Button.H>
 #include <FL/Fl_Int_Input.H>
 #include <FL/Fl_Float_Input.H>
+#include <FL/Fl_Value_Slider.H>
 #include <FL/Fl.H>
 #include <FL/fl_ask.H>
 
@@ -18,11 +19,15 @@ extern "C" {
 
 typedef enum {observed_pdist, Poisson_pdist, Kimura_pdist} protein_distances;
 typedef enum {observed_ndist, Jukes_Cantor, Kimura2P, HKY, LogDet, Ka, Ks} nucleotide_distances;
+typedef enum {more_thorough, less_thorough, rearrange_best_tree} dnapars_S_option;
 
 allseq *view_to_allseq(SEA_VIEW *view, int remove_all_gaps);
 void gaps_as_unknown_states(allseq *phyml_seqs, int protein);
 void free_after_view_to_allseq(allseq *phyml_seqs);
-void initialize_random_series(void);
+//void initialize_random_series(void);
+extern "C" {
+  double next_random(void);
+  }
 void bootstrap_weights(allseq *seqs, int use_codons);
 void savedistinfile(matrix *mat, const char *dirname, const char *outfname);
 matrix *calc_dist_matrix(allseq *seqs, int distkind, int protein, char **lwlseqs, int in_bootstrap, char **p_err_mess);
@@ -47,13 +52,14 @@ int freq_obs(char* seq1, char* seq2, double* freq, int lgseq, int *wght);
 int informative(char** sequ, int nb);
 void jumble_fct(char** seq, char** name, int notu);
 char *run_parsimony_method(SEA_VIEW *view, int nogaps, const char *gapsunknown, int nreplicates, int njumbles, 
-			   int maxbest, int user_tree, Fl_Box *w_count,
-			   const char **pprogname, int no_gui, void (*alert)(const char *,...) );
+			   int maxbest, int user_tree, Fl_Box *w_count, dnapars_S_option s_option,
+			   const char **pprogname, float consensus_threshold, int no_gui, void (*alert)(const char *,...) );
 void parsimony_dialog(SEA_VIEW *view);
 char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-	int *pcount, int maxbest, Fl_Widget *w_count, int no_gui);
+	int *pcount, int maxbest, Fl_Widget *w_count, dnapars_S_option s_option, int no_gui, float consensus_threshold);
 char *parsimony_with_bootstrap(allseq *tmpallseq, char **pars_seqs, char **pars_names, int n_otu, int njumbles, 
-	int protein, int *psteps, int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done, int no_gui);
+	int protein, int *psteps, int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done, dnapars_S_option s_option, int no_gui,
+			       float consensus_threshold);
 double *vector(long nl, long nh);
 void free_vector(double *v, long nl, long nh);
 int ludcmp(double **a, int n, int *indx, double *d);
@@ -75,7 +81,6 @@ extern int init_add_bootstrap(char *full_tree);
 extern char *get_res_value(const char *name, const char *def_value);
 extern int process_one_replicate(char *one_tree, float w);
 extern char *finish_add_bootstrap(int replicates);
-extern void make_strict_consensus(char *tree);
 extern int compare_newick_with_names(const char *tree, char **names, int notu, char **pname);
 extern const char *make_binary_or_unrooted(char *arbre);
 extern int save_resources(void);
@@ -83,24 +88,47 @@ extern int set_res_value(const char *name, const char *value);
 extern char *create_tmp_filename(void);
 extern void delete_tmp_filename(const char *base_fname);
 extern int is_view_valid(SEA_VIEW *view);
-char *parsimony(char** seq, char** seqname, int notu, int* steps, char* toevaluate, int arg_maxtrees, 
-		int *bt_weights, bool protein, int no_gui);
+char *parsimony(char** seq, char** seqname, int notu, int njumbles, int* steps, char* toevaluate, int arg_maxtrees, 
+		int *bt_weights, bool protein, dnapars_S_option s_option, int no_gui, Fl_Widget *w_count);
 extern int fl_create_thread(void (*f) (void *), void* p, unsigned);
 extern char *argname(int argc, char *argv[], const char *arg);
 extern float argval(int argc, char *argv[], const char *arg, float defval);
 extern bool isarg(int argc, char *argv[], const char *arg);
 extern SEA_VIEW *cmdline_read_input_alignment(int argc, char **argv);
+extern void init_consensus_calculation(int n);
+extern int process_one_consensus_item(char *one_tree, char **seqnames);
+extern int after_consensus_items(float threshold, int item_count, int ***prows);
+extern char *compute_consensus_tree(char **seqnames, int **rows, int branch_count);
 extern "C" {
   void prefastlwl(void);
   void loadrl(void);
   int fastlwl(char **seq, int nbseq, int lgseq, double **ka, double **ks, 
 					double **vka, double **vks, int* sat1, int* sat2, int *wght);
-  char* dnapars(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights); 
-  char* protpars(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights); 
+  char* dnapars(char** seq, char** seqname, int notu, int njumbles, int *jumble_no, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, dnapars_S_option s_option); 
+  char* protpars(char** seq, char** seqname, int notu, int njumbles, int *jumble_no, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, dnapars_S_option s_option); 
+  char *create_tmp_filename_from_C(void);
+  FILE *fl_fopen_from_C(const char *fname, const char *mode);
+  int fl_unlink_from_C(const char*fname);
+  void awake_from_C(void);
 }
 
 int tree_build_interrupted;
 
+char *create_tmp_filename_from_C(void)
+{
+  return   create_tmp_filename();
+}
+
+FILE *fl_fopen_from_C(const char *fname, const char *mode)
+{
+  return fl_fopen(fname, mode);
+}
+
+int fl_unlink_from_C(const char*fname)
+{
+  return fl_unlink(fname);
+}
+
 void trees_callback(Fl_Widget *ob, void *data)
 /* 0:Distance  1:PhyML 2:import
  */
@@ -1186,8 +1214,15 @@ allseq *view_to_allseq(SEA_VIEW *view, int remove_all_gaps)
 		p = phyml_seqs->c_seq[j]->state;
 		if(!view->protein) { while((p = strchr(p, 'U')) != NULL) *p = 'T'; }
 		else { while((p = strchr(p, '*')) != NULL) *p = '-'; }//replace stops by gaps
-		j++;
+		if (!view->protein) { // remove non-nucleotide characters
+		  p = phyml_seqs->c_seq[j]->state;
+		  while (*p) {
+		    if (strchr("ABCDGHKMNRSTUVWXY-", *p) == NULL) *p = 'N';
+		    p++;
+		  }
 		}
+		j++;
+	}
 	//remove gap-only or gap-with sites
 	for( j = 0; j < phyml_seqs->clean_len; j++) {
 		if(remove_all_gaps) {//remove any gap-containing site
@@ -1234,7 +1269,7 @@ void free_after_view_to_allseq(allseq *phyml_seqs)
 	free(phyml_seqs);
 }
 
-void initialize_random_series(void)
+/*void initialize_random_series(void)
 {
   static int first = TRUE;
   unsigned seed;
@@ -1250,18 +1285,45 @@ void initialize_random_series(void)
 #endif
     srand(seed);
   }
+}*/
+
+
+double next_random(void)
+{
+  static int first = TRUE;
+  unsigned seed;
+  if (first) {
+    first = FALSE;
+#ifdef WIN32
+    SYSTEMTIME timing;
+    GetSystemTime(&timing);
+    seed = (unsigned)(timing.wMilliseconds + 1000*(timing.wSecond + 60*(timing.wMinute + 60*(timing.wHour + 24*timing.wDay))));
+    srand(seed);
+#else
+    struct tms timing;
+    seed = (unsigned)times(&timing);
+    srandom(seed);
+#endif
+  }
+  
+#ifdef WIN32
+  return rand() / (RAND_MAX +(double)1.);
+#else
+  return random() / (RAND_MAX +(double)1.);
+#endif
 }
 
 void bootstrap_weights(allseq *seqs, int use_codons)
 {
 	phydbl buff;
 	int j, position;
-	initialize_random_series();
+	//initialize_random_series();
 	memset(seqs->wght, 0, seqs->clean_len * sizeof(int));
 	if(!use_codons) {//bootstrap sites
 		for(j = 0; j < seqs->clean_len; j++) {
-			buff  = rand();
-			buff /= (RAND_MAX+1.);
+			/*buff  = rand();
+			buff /= (RAND_MAX+1.);*/
+		  buff = next_random();
 			buff *= seqs->clean_len;
 			position = (int)floor(buff);
 			seqs->wght[position] += 1;
@@ -1271,8 +1333,9 @@ void bootstrap_weights(allseq *seqs, int use_codons)
 		int ncodons = (seqs->clean_len/3);
 		int l3 = 3*ncodons;
 		for(j = 0; j < l3; j += 3) {
-			buff  = rand();
-			buff /= (RAND_MAX+1.);
+			/*buff  = rand();
+			buff /= (RAND_MAX+1.);*/
+		  buff = next_random();
 			buff *= ncodons;
 			position = (int)floor(buff);
 			seqs->wght[3 * position] += 1;//only 0, 3, 6,... have weights
@@ -1696,16 +1759,17 @@ int informative(char** sequ, int nb)
 }
 
 
-void jumble_fct(char** seq, char** name, int notu)
+/*void jumble_fct(char** seq, char** name, int notu)
 {
 	int i, j, rando;
 	double buff;
 	char *tmp;
-	initialize_random_series();
+	//initialize_random_series();
 	for(i = 0; i<notu - 1; i++){
 		j = notu - i - 1;
-		buff  = rand();
-		buff /= (RAND_MAX+1.);
+		//buff  = rand();
+		//buff /= (RAND_MAX+1.);
+		buff = next_random();
 		buff *= j + 1;
 		rando = (int)floor(buff);
 		if(j != rando) {
@@ -1717,11 +1781,12 @@ void jumble_fct(char** seq, char** name, int notu)
 			name[rando] = tmp;
 			}
 		}
-}
+}*/
+
 
 char *run_parsimony_method(SEA_VIEW *view, int nogaps, const char *gapsunknown, int nreplicates, int njumbles, 
-			   int maxbest, int user_tree, Fl_Box *w_count,
-			   const char **pprogname, int no_gui, void (*alert)(const char *,...) )
+			   int maxbest, int user_tree, Fl_Box *w_count, dnapars_S_option s_option,
+			   const char **pprogname, float consensus_threshold, int no_gui, void (*alert)(const char *,...) )
 {
   char *tree = 0, *final_tree = 0;
   int i, steps, info, count, replicates_done;
@@ -1745,7 +1810,7 @@ char *run_parsimony_method(SEA_VIEW *view, int nogaps, const char *gapsunknown,
   }
   if (nreplicates) {
     tree = parsimony_with_bootstrap(tmpallseqs, pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, 
-				    view->protein, &steps, maxbest, nreplicates, w_count, &replicates_done, no_gui);
+				    view->protein, &steps, maxbest, nreplicates, w_count, &replicates_done, s_option, no_gui, consensus_threshold);
     if (tree) {
       final_tree = (char *)malloc(strlen(tree) + 150);
       sprintf(final_tree, "[%s, bootstrap with %d replic., %d steps, %d sites (%d informative)%s]%s",
@@ -1769,7 +1834,7 @@ char *run_parsimony_method(SEA_VIEW *view, int nogaps, const char *gapsunknown,
       input_tree = (char *)realloc(input_tree, strlen(input_tree) + 4 * tmpallseqs->n_otu + 6 );
       if( make_binary_or_unrooted(input_tree) != NULL) return NULL;
       strcat(input_tree, ";");
-      tree = parsimony(pars_seqs, pars_names, tmpallseqs->n_otu, &steps, input_tree, maxbest, NULL, view->protein, no_gui);
+      tree = parsimony(pars_seqs, pars_names, tmpallseqs->n_otu, 0, &steps, input_tree, maxbest, NULL, view->protein, s_option, no_gui, NULL);
       free(input_tree);
       if(tree == NULL) {
 	alert("Error in user-tree");
@@ -1783,11 +1848,20 @@ char *run_parsimony_method(SEA_VIEW *view, int nogaps, const char *gapsunknown,
   }
   else {
     tree = parsimony_with_jumble(pars_seqs, pars_names, tmpallseqs->n_otu, njumbles, view->protein, 
-				 &steps, &count, maxbest, w_count, no_gui);
+				 &steps, &count, maxbest, w_count, s_option, no_gui, consensus_threshold);
+    if (tree && count > 1 && !view->protein) { /* with DNA, fit branch lengths to consensus of most parsimonious trees */
+      char *input_tree = tree;
+      int steps2; /* don't use the steps of this consensus tree because it can be larger than parsimonious steps */
+      tree = parsimony(pars_seqs, pars_names, tmpallseqs->n_otu, 0, &steps2, input_tree, maxbest, NULL, false, s_option, no_gui, NULL);
+      free(input_tree);
+      }
     if (tree) {
+      char aux[30];
       final_tree = (char *)malloc(strlen(tree) + 150);
+      if (count > 1) sprintf(aux, " (%d%% consensus)", int(consensus_threshold*100 + 0.5));
+      else *aux = 0;
       sprintf(final_tree, "[%s, %d best trees%s, %d steps, %d sites (%d informative)%s]%s",
-	      *pprogname, count, count > 1 ? " (strict consensus)" : "", steps, tmpallseqs->clean_len, info, gapsunknown, tree);
+	      *pprogname, count, aux, steps, tmpallseqs->clean_len, info, gapsunknown, tree);
     }
   }
 way_out:
@@ -1801,15 +1875,15 @@ way_out:
 
 void parsimony_dialog(SEA_VIEW *view)
 {
-	int njumbles, i, count, maxbest, started = 0, use_bootstrap, 
+	int njumbles, i, count, maxbest = 100, started = 0, use_bootstrap, 
 		nreplicates, user_tree, nogaps; 
 	char *final_tree;
 	const char *p, *progname;
+  dnapars_S_option s_option = more_thorough;
   
 	tree_build_interrupted = 0;
-	Fl_Window *w = new Fl_Window(255, 225);
+	return_window *w = new return_window(255, view->protein ? 225: 300, "Parsimony analysis");
 	w->set_modal();
-	w->label("Parsimony analysis");
 	Fl_Check_Button *w_dojumble = new Fl_Check_Button(2, 5, 170, 20, "Randomize seq. order");
 	w_dojumble->value(0);
 	Fl_Int_Input *w_jumbles = new Fl_Int_Input(w_dojumble->x() + w_dojumble->w() + 5, w_dojumble->y(), 30, 20, "times");
@@ -1821,11 +1895,36 @@ void parsimony_dialog(SEA_VIEW *view)
 														 "Gaps as unknown states");
 	w_gapsunknown->value(0);
 	w_gapsunknown->deactivate();
-	Fl_Int_Input *w_best = new Fl_Int_Input(195, w_gapsunknown->y() + w_gapsunknown->h() + 5, 50, 20, "Equally best trees retained");
-	w_best->align(FL_ALIGN_LEFT);
-	w_best->value("10000");
-	Fl_Round_Button *w_dobootstrap = new Fl_Round_Button(2, w_best->y() + w_best->h() + 5,
-														115,20, "Bootstrap with");
+  Fl_Widget *witness = w_gapsunknown;
+  Fl_Round_Button *thorough_b, *less_thorough_b;
+  Fl_Int_Input *w_best;
+  if (!view->protein) {
+    Fl_Group *g = new Fl_Group(2, witness->y() + witness->h() + 5, w->w()-4, 70);
+    g->box(FL_DOWN_FRAME);
+    thorough_b = new Fl_Round_Button(7, witness->y() + witness->h() + 5, 180, 20, "More thorough tree search");
+    thorough_b->type(FL_RADIO_BUTTON);
+    less_thorough_b = new Fl_Round_Button(7, thorough_b->y() + thorough_b->h() + 5, 180, 20, "Less thorough tree search");
+    less_thorough_b->type(FL_RADIO_BUTTON);
+    Fl_Round_Button *rearrange_best_b = new Fl_Round_Button(7, less_thorough_b->y() + less_thorough_b->h() + 5, 180, 20, "Rearrange on best tree");
+    rearrange_best_b->type(FL_RADIO_BUTTON);
+    rearrange_best_b->activate();
+    thorough_b->setonly();
+    g->end();
+    w_best = new Fl_Int_Input(195, rearrange_best_b->y() + rearrange_best_b->h() + 5, 50, 20, "Equally best trees retained");
+    w_best->align(FL_ALIGN_LEFT);
+    w_best->value("10000");
+    witness = w_best;
+  }
+  Fl_Value_Slider* slider = new Fl_Value_Slider(2, witness->y() + witness->h() + 25, w->w()-4, 20, "% level for consensus tree building");
+  slider->align(FL_ALIGN_TOP);
+  slider->type(FL_HOR_NICE_SLIDER);
+  slider->box(FL_FLAT_BOX);
+  slider->color(FL_YELLOW);
+  slider->selection_color(FL_RED);
+  slider->bounds(50, 100);
+  slider->value(100);
+  slider->precision(0);
+	Fl_Round_Button *w_dobootstrap = new Fl_Round_Button(2, slider->y() + slider->h() + 10, 115, 20, "Bootstrap with");
 	w_dobootstrap->type(FL_RADIO_BUTTON);
 	Fl_Int_Input *w_btrepl = new Fl_Int_Input(w_dobootstrap->x() + w_dobootstrap->w() + 5, w_dobootstrap->y() ,
 											  40, 20, "replicates");
@@ -1843,6 +1942,7 @@ void parsimony_dialog(SEA_VIEW *view)
 	
 	Fl_Button *w_interrupt = new Fl_Button(w_dobootstrap->x(), w_user->y() + w_user->h() + 5, 60, 20, "Cancel");
 	Fl_Return_Button *w_ok = new Fl_Return_Button(w->w() - 65, w_interrupt->y(), 60, 20, "OK");
+	w->run(w_ok);
 	w_interrupt->callback(interrupt_callback, &tree_build_interrupted);
 	w_ok->callback(interrupt_callback, &started);
 	w->end();
@@ -1862,8 +1962,6 @@ void parsimony_dialog(SEA_VIEW *view)
 	}
 	w_ok->hide();
 	w_interrupt->label("Interrupt");
-	p = w_best->value();
-	sscanf(p, "%d", &maxbest);
 	if(w_dojumble->value()) {
 		p = w_jumbles->value();
 		sscanf(p, "%d", &njumbles);
@@ -1884,13 +1982,19 @@ void parsimony_dialog(SEA_VIEW *view)
 	else {
 		if(user_tree) user_tree = w_choice->value() + 1;
 	}
-
+  if (!view->protein) {
+    if (thorough_b->value()) s_option = more_thorough;
+    else if (less_thorough_b->value()) s_option = less_thorough;
+    else s_option = rearrange_best_tree;
+    p = w_best->value();
+    sscanf(p, "%d", &maxbest);
+    }
 	my_watch_cursor(w);
 
 	final_tree = run_parsimony_method(view, nogaps, gapsunknown, 
 				    use_bootstrap && nreplicates > 1 ? nreplicates : 0, njumbles, 
-					  maxbest, user_tree, w_count, 
-					  &progname, false, fl_alert);
+					  maxbest, user_tree, w_count, s_option,
+					  &progname, slider->value()/100, false, fl_alert);
 
 	fl_reset_cursor(w);
 	delete w;
@@ -1911,40 +2015,25 @@ void parsimony_dialog(SEA_VIEW *view)
 
 
 char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-	int *pcount, int maxbest, Fl_Widget *w_count, int no_gui)
+	int *pcount, int maxbest, Fl_Widget *w_count, dnapars_S_option s_option, int no_gui, float consensus_threshold)
 {
-	int i, steps, newsteps, count = 0;
+	int i, steps, count = 0;
 	char *tree, *newtree, *p, *q;
 	if (njumbles > 0) {
-	  if (w_count) w_count->label("Initial sequence order");
+	  if (w_count) w_count->label("First jumble attempt");
 	  }
-	tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, NULL, protein, no_gui);
+	tree = parsimony(pars_seqs, pars_names, n_otu, njumbles, &steps, NULL, maxbest, NULL, protein, s_option, no_gui, w_count);
 	if (!tree) return NULL;
-	newsteps = steps;
-	for( i = 1; i <= njumbles; i++) {
-		char tmp[40];
-		sprintf(tmp, "Jumble attempt: %d steps: %d", i, newsteps); 
-		if (w_count) w_count->label(tmp); 
-		jumble_fct(pars_seqs, pars_names, n_otu);
-		newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, NULL, protein, no_gui);
-		if (!newtree) break;
-		if(newsteps < steps) {
-			steps = newsteps;
-			free(tree);
-			tree = newtree;
-		}
-		else free(newtree);
-	}
 	p = tree - 1;
 	while((p = strchr(p+1, '\n')) != NULL) count++;//count equally parsimonious trees
-	if(count > 1) {//compute strict consensus of all equally parsimonious trees
+	if(count > 1) {//compute threshold consensus of all equally parsimonious trees
 		p = strchr(tree, '\n');
 		newtree = (char *)malloc(p - tree + 1);
 		memcpy(newtree, tree, p - tree); newtree[p - tree] = 0;
 		p++; i = count; count = 0;
-		init_add_bootstrap(newtree);
+		init_consensus_calculation(n_otu);
 		while(TRUE) {
-			count += process_one_replicate(newtree, 1.);
+			count += process_one_consensus_item(newtree, pars_names);
 			free(newtree);
 			i--;
 			if(i == 0) break;
@@ -1954,8 +2043,11 @@ char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int
 			p = q + 1;
 		}
 		free(tree);
-		tree = finish_add_bootstrap(count);
-		make_strict_consensus(tree);
+		int **rows;
+		int br_count = after_consensus_items(consensus_threshold, count, &rows);
+		tree = compute_consensus_tree(pars_names, rows, br_count);
+		for (i = 0; i < br_count; i++) free(rows[i]);
+		free(rows);
 	}
 	*psteps = steps;
 	*pcount = count;
@@ -1964,40 +2056,38 @@ char *parsimony_with_jumble(char **pars_seqs, char **pars_names, int n_otu, int
 
 
 char *parsimony_with_bootstrap(allseq *tmpallseqs, char **pars_seqs, char **pars_names, int n_otu, int njumbles, int protein, int *psteps,
-			int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done, int no_gui)
+			int maxbest, int replicates, Fl_Box *w_count, int *preplicates_done, dnapars_S_option s_option, int no_gui,
+			       float consensus_threshold)
 {
 	int i, j, steps, newsteps, count = 0, replicates_done = 0;
 	char *tree, *newtree, *p;
 	
 	if (w_count) w_count->label("Full sequence tree"); 
 	tree = parsimony_with_jumble(pars_seqs, pars_names, n_otu, njumbles, protein, 
-								 &steps, &count, maxbest, w_count, no_gui);
+								 &steps, &count, maxbest, NULL, s_option, no_gui, consensus_threshold);
 	if (!tree) return NULL;
+	if (count > 1 && !protein) { /* with DNA, fit branch lengths to strict consensus of most parsimonious trees */
+	  char *input_tree = tree;
+	  /* don't use the steps of this consensus tree because it can be larger than parsimonious steps */
+	  tree = parsimony(pars_seqs, pars_names, n_otu, njumbles, &newsteps, input_tree, maxbest, NULL, false, s_option, no_gui, NULL);
+	  free(input_tree);
+	}
 	*psteps = steps;
-	init_add_bootstrap(tree);
+	init_add_bootstrap(tree); // memorize topology and br lengths of strict consensus of most parsimonious trees
 	for(j = 0; j < replicates; j++) {
-		char tmp[30];
-		sprintf(tmp, "Bootstrap replicate: %d", j + 1); 
-		if (w_count) w_count->label(tmp); 
+		if (w_count) {
+		  char tmp[30];
+		  sprintf(tmp, "Bootstrap replicate: %d", j + 1); 
+		  w_count->label(tmp);
+		}
 		if (tree_build_interrupted) break;
 		bootstrap_weights(tmpallseqs, FALSE);
-		tree = parsimony(pars_seqs, pars_names, n_otu, &steps, NULL, maxbest, tmpallseqs->wght, protein, no_gui);
+		tree = parsimony(pars_seqs, pars_names, n_otu, njumbles, &steps, NULL, maxbest, tmpallseqs->wght, protein, s_option, no_gui, NULL);
 		if (!tree) break;
-		for( i = 1; i <= njumbles; i++) {
-			jumble_fct(pars_seqs, pars_names, n_otu);
-			newtree = parsimony(pars_seqs, pars_names, n_otu, &newsteps, NULL, maxbest, tmpallseqs->wght, protein, no_gui);
-			if (!newtree) break;
-			if(newsteps < steps) {
-				steps = newsteps;
-				free(tree);
-				tree = newtree;
-			}
-			else free(newtree);
-		}
 		p = tree - 1; count = 0;
 		while((p = strchr(p+1, '\n')) != NULL) count++;//count equally parsimonious trees
 		i = count;
-		while(TRUE) {
+		while (TRUE) {
 			p = strchr(tree, '\n');
 			newtree = (char *)malloc(p - tree + 1);
 			memcpy(newtree, tree, p - tree); newtree[p - tree] = 0;
@@ -2021,48 +2111,74 @@ struct pars_struct {
   char** seq;
   char** seqname;
   int notu;
+  int njumbles;
+  int *jumble_no;
   int *steps;
   char* toevaluate;
   int arg_maxtrees;
   int *bt_weights;
+  dnapars_S_option s_option;
   char *result;
   bool protein;
   bool done;
 };
 
+
+void awake_from_C(void)
+{
+  Fl::lock();
+  Fl::awake();
+  Fl::unlock();
+}
+
+
 static void parsimony_thread(pars_struct* args)
 {
-  typedef char* (*pars_f)(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights);
+  typedef char* (*pars_f)(char** seq, char** seqname, int notu, int njumbles, int *jumble_no, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, dnapars_S_option s_option);
   pars_f pf = args->protein ? protpars : dnapars;
-  args->result = (*pf)(args->seq, args->seqname, args->notu, args->steps, args->toevaluate, args->arg_maxtrees, args->bt_weights);
+  args->result = (*pf)(args->seq, args->seqname, args->notu, args->njumbles, args->jumble_no, args->steps, args->toevaluate, args->arg_maxtrees, args->bt_weights, args->s_option);
   Fl::lock();
   args->done = true;
   Fl::awake();
   Fl::unlock();
 }
 
-char *parsimony(char** seq, char** seqname, int notu, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, bool protein, int no_gui)
+char *parsimony(char** seq, char** seqname, int notu, int njumbles, int *steps, char* toevaluate, int arg_maxtrees, int *bt_weights, 
+		bool protein, dnapars_S_option s_option, int no_gui, Fl_Widget *w_count)
 {
   char *p;
+  int jumble_no = 0;
+  int pre_jumble_no = 0;
   tree_build_interrupted = 0;
   if (no_gui) {
-    if (protein) p = protpars(seq, seqname, notu, steps, toevaluate, arg_maxtrees, bt_weights);
-    else  p = dnapars(seq, seqname, notu, steps, toevaluate, arg_maxtrees, bt_weights);
+    if (protein) p = protpars(seq, seqname, notu, njumbles, NULL, steps, toevaluate, arg_maxtrees, bt_weights, s_option);
+    else  p = dnapars(seq, seqname, notu, njumbles, NULL, steps, toevaluate, arg_maxtrees, bt_weights, s_option);
   }
   else {
     pars_struct *args = (pars_struct*)malloc(sizeof(pars_struct));
     args->seq = seq;
     args->seqname = seqname;
     args->notu = notu;
+    args->njumbles = njumbles;
+    args->jumble_no = &jumble_no;
     args->steps = steps;
     args->toevaluate = toevaluate;
     args->arg_maxtrees = arg_maxtrees;
     args->bt_weights = bt_weights;
+    args->s_option = s_option;
     args->result = NULL;
     args->protein = protein;
     args->done = false;
     if ( ! fl_create_thread((void (*) (void *))parsimony_thread, args, 1024) ) {
-      while (!args->done) Fl::wait();
+      while (!args->done) {
+	Fl::wait();
+	if (jumble_no > pre_jumble_no && jumble_no < njumbles && w_count) {
+	  pre_jumble_no = jumble_no;
+	  char tmp[40];
+	  sprintf(tmp, "Jumble attempt: %d steps: %d", jumble_no + 1, *args->steps); 
+	  w_count->label(tmp); 
+	}
+	}
     }
     p = args->result;
     free(args);
@@ -2363,9 +2479,14 @@ void command_line_phylogeny(int argc, char **argv)
   if (isarg(argc, argv, "-parsimony")) { // parsimony method
     const char *progname;
     jumbles = argval(argc, argv, "-jumbles", 0);
+    char *search = argname(argc, argv, "-search");
+    dnapars_S_option s_option;
+    if (search && strcmp(search, "less") == 0) s_option = less_thorough;
+    else if (search && strcmp(search, "best") == 0) s_option = rearrange_best_tree;
+    else s_option = more_thorough;
     tree = run_parsimony_method(view, isarg(argc, argv, "-nogaps"), 
 				isarg(argc, argv, "-gaps_as_unknown") ? ", gaps treated as ?" : "", 
-				replicates, jumbles, 10000, 0, NULL, &progname, true, Fl::fatal);
+				replicates, jumbles, 10000, 0, NULL, s_option, &progname, 1, true, Fl::fatal);
   }
   if (tree) {
     char *outfname = argname(argc, argv, "-o");
