@@ -61,7 +61,7 @@ extern void *check_alloc(int nbrelt, int sizelt);
 extern void my_watch_cursor(Fl_Window *win);
 extern void fl_reset_cursor(Fl_Window *win);
 extern char *ecrit_arbre_parenth_unrooted(FD_nj_plot *fd_nj_plot, struct noeud *root);
-extern double place_midpoint_root(struct noeud *from, struct noeud *racine, int fixed_outgroup);
+extern double place_midpoint_root(struct noeud *from, struct noeud *racine, int notu, int fixed_outgroup);
 extern const char *make_binary_or_unrooted(char *arbre);
 extern int make_binary(char *arbre, char *debut, char *fin, int go_down);
 extern char *nextpar(char *pospar);
@@ -767,21 +767,23 @@ const char *preptree_unrooted(const char *tree, FD_unrooted *data)
 	data->tabtax = (struct noeud **)check_alloc(2*data->notu - 1,sizeof(struct noeud *));
 	if(data->notu > 3) data->branche_noeuds = 
 		(branche_noeud *)check_alloc(data->notu-3, sizeof(branche_noeud));
-	for(i=0; i<2*data->notu - 1; i++) *(data->tabtax+i)=
-		(struct noeud *)check_alloc(1, sizeof(struct noeud));
+	for (i = 0; i < 2*data->notu - 1; i++) {
+	  data->tabtax[i] = (struct noeud *)check_alloc(1, sizeof(struct noeud));
+	  data->tabtax[i]->rank = i;
+	  }
 	loadphylip(arbre, data);
 	free(arbre);
 	if(data->notu == 0) goto erreur;
 	if(!data->rooted) {
 		data->racine = *(data->tabtax+(++num_noeud));
 		if(num_noeud >= 2*data->notu - 1) return("Error: incorrect tree file");
-		data->root_br_l = place_midpoint_root(data->tabtax[0], data->racine, FALSE);
+	  data->root_br_l = place_midpoint_root(data->tabtax[0], data->racine, data->notu, FALSE);
 	}
 	else	{
 		return ("Unexpected rooted tree.");
 	}
 	if(data->notu < 3) return ("Tree should contain at least 3 elements.");
-	return (NULL);
+  return (NULL);
 	
 erreur:
 	return ("Not a valid tree.");
