@@ -1,5 +1,6 @@
 #include "seaview.h"
 #include "pdf_or_ps.h"
+#include "svg.h"
 #include <ctype.h>
 #include <time.h>
 #ifndef WIN32
@@ -60,21 +61,39 @@ char* seaview_file_chooser_save_as(const char* message, const char* fname, SEA_V
 const char *extract_dirname(const char *pathname);
 int printout(SEA_VIEW *view, const char *filename, 
 	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, 
-	     int pdfkindvalue, Fl_Paged_Device::Page_Layout layout);
+	     int pdfkindvalue, Fl_Paged_Device::Page_Layout layout, int svg_width = 0);
 static void color_pdf_display(SEA_VIEW *view, int (*calc_color_function)( int ), char *oneline, 
 			      int widnames, double x, double y, int fontsize, double char_width, double descender,int num, int current);
+static void color_svg_display(SEA_VIEW *view, int (*calc_color_function)( int ), char *oneline, 
+			      int widnames, double x, double y, int fontsize, double char_width);
 static int calc_vary_lines(int *vary_pos, int widpos);
 static void out_vary_pos(int *vary_pos, int widnames, int widpos, int nl, FILE *textfile, double x, double y);
 SEA_VIEW* read_alignment_file(const char *infile);
 SEA_VIEW *cmdline_read_input_alignment(int argc, char **argv);
 void format_conversion(int argc, char **argv);
 char *process_output_options(int argc, char **argv, known_format& out_format, bool& std_output);
+#ifndef NO_PDF
+void printout_cmdline(int argc, char **argv);
+#endif
 
 
 /* external */
 extern char *f_format_names[];
 extern char *f_format_exts[];
 extern int nbr_formats;
+extern float argval(int argc, char *argv[], const char *arg, float defval);
+extern int calc_max_seq_length(int seq_length, int tot_seqs);
+extern int max_protcolors;
+extern char def_stdcolorgroups[];
+extern int def_protcolors_rgb[];
+char *get_res_value(const char *name, const char *def_value);
+extern int prep_custom_colors(int *colors, char *customcolors, int max_colors);
+extern color_choice prep_aa_color_code(char *list_std, char *list_alt, 
+				       int maxprotcolors, int *numb_stdprotcolors, int *numb_altprotcolors);
+extern void load_resources(const char *progname);
+extern void allonge_seqs(char **seq, int totseqs, int maxlen, int *eachlength,
+			 int tot_comment_lines, char **comment_line, char **pregion_line);
+extern const char *progname;
 #if defined(__APPLE__)
 extern const char *MG_GetBundleResourcesDir(void);
 #else
@@ -1589,7 +1608,7 @@ Fl_Native_File_Chooser *chooser = new Fl_Native_File_Chooser();
 
 chooser->type(Fl_Native_File_Chooser::BROWSE_SAVE_FILE);   
 chooser->options(Fl_Native_File_Chooser::SAVEAS_CONFIRM | chooser->options());
-#if HAVE_USE_FILTER_EXT || defined(__APPLE__)
+#if SEAVIEW_FLTK_VERSION >= 133 || defined(__APPLE__)
 chooser->options(Fl_Native_File_Chooser::USE_FILTER_EXT | chooser->options());
 #endif
 chooser->title(message);  
@@ -1630,11 +1649,11 @@ const char *extract_dirname(const char *pathname)
 
 int printout(SEA_VIEW *view, const char *filename, 
 	     int fontsize, int block_size, Fl_Paged_Device::Page_Format pageformat, int vary_only, int ref0, 
-	     int pdfkindvalue, Fl_Paged_Device::Page_Layout layout)
+	     int pdfkindvalue, Fl_Paged_Device::Page_Layout layout, int svg_width)
 {
   int num, i, j, k, current, max_seq_length, fin, curr_lines, widnames, 
-  res_per_line, nl, firstpage, lines_per_page, use_pdf;
-  Fl_PDF_or_PS_File_Device *surface;
+  res_per_line, nl, firstpage, lines_per_page, use_pdf, use_svg, top_margin;
+  Fl_Surface_Device *surface;
   FILE *textfile = NULL;
   time_t heure;
   static char unnamed[] = "<unnamed>";
@@ -1642,79 +1661,115 @@ int printout(SEA_VIEW *view, const char *filename,
   int lettre, char_per_line;
   short *vary_need = NULL;
   int *vary_pos; /* rang ds alignement de la colonne imprime */
-  char oneline[500];
+  char *p, oneline[500];
   int (*calc_color_function)(int);
-  double	char_width, descender, margin = 25;
+  double	char_width = fontsize/2, descender, margin = 25;
   if(view->tot_seqs == 0) return 0;
   if(view->protein) calc_color_function = get_color_for_aa;
   else  calc_color_function = get_color_for_base;
-  use_pdf = (pdfkindvalue != TEXT_ONLY);
-  if(use_pdf) {
+  use_pdf = (pdfkindvalue == PDF_COLOR || pdfkindvalue == PDF_BW);
+  use_svg = (pdfkindvalue == SVG);
+  max_seq_length = 0; widnames = 0;
+  for (i=0; i < view->tot_seqs; i++) {
+    if (view->each_length[i] > max_seq_length) max_seq_length = view->each_length[i];
+    if ( ( fin=strlen(view->seqname[i]) ) > widnames) widnames = fin;
+  }
+  widnames += 2;
+  if (use_pdf) {
     surface = new Fl_PDF_or_PS_File_Device();
-    if (surface->begin_document(filename, pageformat, layout)) {
+    if (((Fl_PDF_or_PS_File_Device*)surface)->begin_document(filename, pageformat, layout)) {
       delete surface;
       return 0;
     }
   }
+  else if(use_svg) {
+    FILE *out = fl_fopen(filename, "w");
+    if (out == NULL) exit(1);
+#ifndef NO_PDF
+    Fl_SVG_PDF_width_Graphics_Driver *pdf_d = new Fl_SVG_PDF_width_Graphics_Driver(NULL);
+    pdf_d->font(FL_COURIER, fontsize);
+    char_width = pdf_d->width("X", 1);
+    delete pdf_d;
+#endif
+    margin = char_width;
+    char_per_line = (int)( (svg_width - 2*margin) / char_width + 0.5);
+    fin = (char_per_line - widnames + 1) / (block_size + 1);
+    if (fin < 1) { /* garde fou */
+      fin = 1; block_size = char_per_line - widnames;
+    }
+    res_per_line = fin * block_size;
+    int nl = (max_seq_length + res_per_line - 1) /res_per_line;
+    int totl = nl * (view->tot_seqs + 2) + 2;
+    surface = new Fl_SVG_File_Surface(svg_width, totl * fontsize, out, true);
+    surface->set_current();
+    fl_color(210,210,210); // draw grey background
+    fl_rectf(0,0,svg_width, totl * fontsize);
+    }
   else {
     textfile = fopen(filename, "w");
     if(textfile == NULL) return TRUE;
   }
 #ifndef NO_PDF
   jmp_buf* jbuf;
-  if (use_pdf) surface->surface_try(&jbuf);
+  if (use_pdf) ((Fl_PDF_or_PS_File_Device*)surface)->surface_try(&jbuf);
   if ( (!use_pdf) || (setjmp(*jbuf) == 0)) {
 #endif
-    if(use_pdf) {
+    if (use_pdf) {
       surface->driver()->font(FL_COURIER, fontsize);
       char_width = fl_width("X");
       int pwidth, pheight;
-      surface->printable_rect(&pwidth, &pheight);
+      ((Fl_PDF_or_PS_File_Device*)surface)->printable_rect(&pwidth, &pheight);
       int l, r, t, b;
-      surface->margins(&l, &t, &r, &b);
+      ((Fl_PDF_or_PS_File_Device*)surface)->margins(&l, &t, &r, &b);
       margin -= l;
       char_per_line = (int)((pwidth - 2*margin) / char_width + 0.5);
       lines_per_page = (int)((pheight - 2*margin) / fontsize + 0.5);
       descender = fl_descent();
+      top_margin = margin;
     }
+    else if (use_svg) {
+      surface->driver()->font(FL_COURIER, fontsize);
+      lines_per_page = 10000000;
+      top_margin = fontsize;
+      }
     else char_per_line = 90;
     firstpage = TRUE;
     
     if(ref0 < 0) vary_only = FALSE;
     time(&heure);
     sprintf(oneline,"Alignment: %s", view->masename == NULL ? unnamed : PREPARE_LABEL(view->masename) );
-    if(use_pdf) {
-      surface->start_page();
-      surface->origin(0, fontsize);
-      surface->driver()->font(FL_COURIER, fontsize);
-      fl_draw(oneline, margin, margin); 
+    if (use_pdf) {
+      ((Fl_PDF_or_PS_File_Device*)surface)->start_page();
+      ((Fl_PDF_or_PS_File_Device*)surface)->origin(0, fontsize);
+      fl_draw(oneline, margin, top_margin); 
     }
-    else {fputs(oneline, textfile); fputs("\n", textfile);}
+    else if(!use_svg) {fputs(oneline, textfile); fputs("\n", textfile);}
     curr_lines = 1;
     if(vary_only) {
       const char fixed[] = "Displaying variable sites only.";
-      if(use_pdf) fl_draw(fixed, margin, margin + curr_lines * fontsize); 
+      if (use_pdf || use_svg) fl_draw(fixed, margin, top_margin + curr_lines * fontsize); 
       else {fputs(fixed, textfile); fputs("\n", textfile);}
       ++curr_lines;
     }
-    sprintf(oneline,"Seaview [blocks=%d fontsize=%d %s%s] on %s",
+    if (use_pdf) {
+      sprintf(oneline,"Seaview [blocks=%d fontsize=%d %s%s] on %s",
 	    block_size, fontsize, pageformat == Fl_Paged_Device::A4 ? "A4" : "LETTER", 
-	    layout == Fl_Paged_Device::LANDSCAPE ? "-landscape" : "",
-	    ctime(&heure));
-    if(use_pdf) {
-      char *p = strchr(oneline, '\n'); if (p) *p = 0;
-      fl_draw(oneline, margin, margin + curr_lines * fontsize); 
+	    layout == Fl_Paged_Device::LANDSCAPE ? "-landscape" : "", ctime(&heure));
+      p = strchr(oneline, '\n'); if (p) *p = 0;
+      fl_draw(oneline, margin, top_margin + curr_lines * fontsize); 
+      curr_lines += 2;
+    }
+    else if(use_svg) {
+      sprintf(oneline,"created by Seaview on %s", ctime(&heure));
+      p = strchr(oneline, '\n'); if (p) *p = 0;
+      fl_color(FL_BLACK);
+      fl_draw(oneline, margin, top_margin + curr_lines * fontsize); 
+      curr_lines += 1;
     }
     else {
       fputs("Seaview text-only output\n", textfile);
+      curr_lines += 2;
     }
-    curr_lines += 2;
-    max_seq_length = 0; widnames = 0;
-    for(i=0; i < view->tot_seqs; i++) {
-      if(view->each_length[i] > max_seq_length) max_seq_length = view->each_length[i];
-      if( ( fin=strlen(view->seqname[i]) ) > widnames) widnames = fin;
-    }
-    widnames += 2;
     if(vary_only) {
       vary_need = (short *)calloc(max_seq_length, sizeof(short));
       if(vary_need == NULL) return TRUE;
@@ -1751,14 +1806,14 @@ int printout(SEA_VIEW *view, const char *filename,
 	nl = calc_vary_lines(vary_pos,  k);
       }
       if( use_pdf && (!firstpage) && (curr_lines + view->tot_seqs + nl > lines_per_page)) {
-	surface->end_page();
-	surface->start_page();
-	surface->origin(0, fontsize);
+	((Fl_PDF_or_PS_File_Device*)surface)->end_page();
+	((Fl_PDF_or_PS_File_Device*)surface)->start_page();
+	((Fl_PDF_or_PS_File_Device*)surface)->origin(0, fontsize);
 	surface->driver()->font(FL_COURIER, fontsize);
 	curr_lines = 0;
       }
       if(vary_only) {
-	out_vary_pos(vary_pos, widnames, k, nl, textfile, margin, margin + curr_lines * fontsize);
+	out_vary_pos(vary_pos, widnames, k, nl, textfile, margin, top_margin + curr_lines * fontsize);
 	curr_lines += nl;
       }
       else	{
@@ -1766,7 +1821,7 @@ int printout(SEA_VIEW *view, const char *filename,
 	fin = strlen(num_line);
 	memmove(num_line + widnames - fin + 1, num_line, fin+1);
 	if(fin <= widnames) memset(num_line, ' ', widnames - fin + 1);
-	if( use_pdf) fl_draw(num_line, margin, margin + curr_lines * fontsize); 
+	if( use_pdf || use_svg) fl_draw(num_line, margin, top_margin + curr_lines * fontsize); 
 	else {fputs(num_line, textfile);fputs("\n",textfile);}
 	++curr_lines;
       }
@@ -1822,27 +1877,29 @@ int printout(SEA_VIEW *view, const char *filename,
 	oneline[k] = 0;
 	if(!view->allow_lower) majuscules(oneline + widnames);
 	if(use_pdf && (curr_lines >= lines_per_page)) {
-	  surface->end_page();
-	  surface->start_page();
-	  surface->origin(0, fontsize);
+	  ((Fl_PDF_or_PS_File_Device*)surface)->end_page();
+	  ((Fl_PDF_or_PS_File_Device*)surface)->start_page();
+	  ((Fl_PDF_or_PS_File_Device*)surface)->origin(0, fontsize);
 	  surface->driver()->font(FL_COURIER, fontsize);
 	  curr_lines = 0;
 	}
-	if(!use_pdf) {
+	if(!use_pdf && !use_svg) {
 	  fputs(oneline, textfile); fputs("\n", textfile);
 	}
 	else if(pdfkindvalue == PDF_BW) {
-	  fl_draw(oneline, margin, margin + curr_lines * fontsize);
+	  fl_draw(oneline, margin, top_margin + curr_lines * fontsize);
 	}
 	else {
-	  color_pdf_display(view, calc_color_function, oneline, widnames, margin, 
-			    margin + curr_lines * fontsize, 
+	  if (use_pdf) color_pdf_display(view, calc_color_function, oneline, widnames, margin, 
+			    top_margin + curr_lines * fontsize, 
 			    fontsize, char_width, descender, num, current);
+	  else color_svg_display(view, calc_color_function, oneline, widnames, margin, 
+				 top_margin + curr_lines * fontsize, fontsize, char_width);
 	}
 	++curr_lines;
 	firstpage = FALSE;
       }
-      if(!use_pdf) {
+      if(!use_pdf && !use_svg) {
 	fputs("\n", textfile);
       }
       else if(curr_lines + 1 <= lines_per_page) {
@@ -1851,17 +1908,23 @@ int printout(SEA_VIEW *view, const char *filename,
       if( ! vary_only ) current += res_per_line;
     }
     if(use_pdf) {
-      surface->end_page();
-      surface->end_job();
+      ((Fl_PDF_or_PS_File_Device*)surface)->end_page();
+      ((Fl_PDF_or_PS_File_Device*)surface)->end_job();
     }
+    else if(use_svg) {
+      fl_color(FL_GRAY);
+      fl_line_style(0, 3);
+      fl_rect(1,1, ((Fl_SVG_File_Surface*)surface)->width()-2, ((Fl_SVG_File_Surface*)surface)->height()-2);
+      ((Fl_SVG_File_Surface*)surface)->end();
+      }
     else fclose(textfile);
 #ifndef NO_PDF
   } /* end of PDF_TRY */
-  if (use_pdf && surface->surface_catch()) {
-    surface->error_catch();
+  if (use_pdf && ((Fl_PDF_or_PS_File_Device*)surface)->surface_catch()) {
+    ((Fl_PDF_or_PS_File_Device*)surface)->error_catch();
   }
 #endif
-  if (use_pdf) delete surface;
+  if (use_pdf || use_svg) delete surface;
   return FALSE;
 }
 
@@ -1891,7 +1954,7 @@ static void color_pdf_display(SEA_VIEW *view, int (*calc_color_function)( int ),
     fl_color(view->curr_colors[c]);
     for(xx = x + widnames*char_width, p = clines[c] + widnames; *p != 0; p++, xx += char_width) {
       if(*p == ' ') continue;
-      fl_rectf(xx, y - fontsize + descender, char_width, fontsize);
+      fl_rectf(xx, y - fontsize + descender, char_width+1, fontsize);
     }
   }
   fl_color(FL_BLACK);
@@ -1900,6 +1963,29 @@ static void color_pdf_display(SEA_VIEW *view, int (*calc_color_function)( int ),
   free(clines);
 }
 
+static void color_svg_display(SEA_VIEW *view, int (*calc_color_function)( int ), char *oneline, 
+			      int widnames, double x, double y, int fontsize, double char_width)
+{
+  double  xx;
+  int c, l;
+  char *p;
+  
+  l = strlen(oneline);
+  char *aux = (char*)malloc(l+1);
+  fl_font(FL_COURIER_BOLD, fontsize);
+  xx = x + widnames*char_width;
+  for (c = 0; c < view->numb_gc; c++) {
+    memset(aux, ' ', l-widnames);
+    for (p = oneline+widnames; p < oneline+l; p++) {
+      if (calc_color_function(*p) == c) aux[p-(oneline+widnames)] = *p;
+      }
+    fl_color(view->curr_colors[c]);
+    fl_draw(aux, l - widnames, xx, y);
+  }
+  fl_font(FL_COURIER, fontsize);
+  fl_color(FL_BLACK);
+  fl_draw(oneline, widnames, x, y);
+}
 
 static int calc_vary_lines(int *vary_pos, int widpos)
 {
@@ -2048,6 +2134,9 @@ SEA_VIEW *cmdline_read_input_alignment(int argc, char **argv)
     fputs("\n", stderr);
     exit(1);
   }
+  view->max_seq_length = calc_max_seq_length(view->seq_length, view->tot_seqs);
+  allonge_seqs(view->sequence, view->tot_seqs, view->max_seq_length, view->each_length, 
+	       view->tot_comment_lines, view->comment_line, NULL);
   return view;
 }
 
@@ -2295,3 +2384,75 @@ char *process_output_options(int argc, char **argv, known_format& out_format, bo
   return outfile;
 }
 
+
+#ifndef NO_PDF
+void printout_cmdline(int argc, char **argv)
+{
+  bool std_output = false;
+  SEA_VIEW *view = cmdline_read_input_alignment(argc, argv);
+  if (view == NULL) {
+    fprintf(stderr, "No sequence found in %s", argv[argc-1]);
+    exit(1);
+  }  
+  load_resources(progname);
+  //protein colors
+  static char stdcolorgroups[50], customprotcolors[300];
+  strcpy(stdcolorgroups, get_res_value("stdcolorgroups", def_stdcolorgroups));
+  strcpy(customprotcolors, get_res_value("protcolors", ""));
+  int *protcolors = (int *)malloc(max_protcolors * sizeof(int));
+  int *def_protcolors = (int *)malloc(max_protcolors * sizeof(int));
+  for(int i=0; i<max_protcolors; i++) def_protcolors[i] = 
+    fl_rgb_color(def_protcolors_rgb[3*i], def_protcolors_rgb[3*i+1], def_protcolors_rgb[3*i+2]);
+  
+  memcpy(protcolors, def_protcolors, max_protcolors * sizeof(int) );
+  int cur_protcolors = prep_custom_colors(protcolors, customprotcolors, max_protcolors);
+  static int dnacolors[] = { FL_BLACK, FL_RED, FL_DARK_GREEN, FL_YELLOW, FL_BLUE };
+  /* process resource-read stdcolorgroups and altcolorgroups */
+  prep_aa_color_code(stdcolorgroups, (char*)"", 
+					 cur_protcolors, &view->numb_stdprotcolors, &view->numb_altprotcolors);
+  
+  view->max_seq_length = calc_max_seq_length(view->seq_length, view->tot_seqs);
+  if (view->protein) {
+    view->numb_gc = view->numb_stdprotcolors;
+    view->curr_colors = view->stdprotcolors = protcolors;
+    protcolors[0] = FL_BLACK;
+    }
+  else {
+    view->numb_gc = view->numb_dnacolors = 5;
+    view->curr_colors = view->dnacolors = dnacolors;
+    }
+  known_format out_format = view->format_for_save;
+  char *outfile = process_output_options(argc, argv, out_format, std_output);
+  int fontsize = (int)argval(argc, argv, "-fontsize", 10);
+  int blocksize = (int)argval(argc, argv, "-blocksize", 10);
+#ifdef NO_PDF
+  int use_svg = false;
+  int svg_width = 0;
+#else
+  int use_svg = isarg(argc, argv, "-svg");
+  int svg_width = (int)argval(argc, argv, "-svg", 1000);
+#endif
+  int landscape = isarg(argc, argv, "-landscape");
+  int letter = isarg(argc, argv, "-letter");
+  printout(view, outfile, 
+	    fontsize,  blocksize, 
+	   letter ? Fl_Paged_Device::LETTER : Fl_Paged_Device::A4, 
+	   false, -1, 
+	   use_svg ? SVG : PDF_COLOR, 
+	   landscape ? Fl_Paged_Device::LANDSCAPE : Fl_Paged_Device::PORTRAIT, 
+	   svg_width);
+  if (std_output) {
+    char line[200];
+    FILE *in = fopen(outfile, "r");
+    if (in) {
+      while (fgets(line, sizeof(line), in) != NULL) {
+	fputs(line, stdout);
+      }
+      fclose(in);
+      fflush(stdout);
+    }
+    delete_tmp_filename(outfile);
+  }
+  exit(0);
+}
+#endif // ! NO_PDF
