@@ -10,9 +10,8 @@
 void my_bell(void);
 char *get_prog_dir(void);
 char **getutf8args(int *pargc);
+int mysystem(const char *cmd);
 
-/* extern vars & functions */
-extern char *wchartoutf8(const wchar_t *in);
 
 Fl_Copy_Surface::Fl_Copy_Surface(int w, int h) : Fl_Surface_Device(new Fl_GDI_Graphics_Driver())
 {
@@ -65,7 +64,10 @@ char *get_prog_dir(void)
 	int l;
 	
 	if(dir == NULL) {
-		line = wchartoutf8(GetCommandLineW());
+		WCHAR *wp = GetCommandLineW();
+		l = 4*wcslen(wp)+1;
+		line = new char[l];
+		fl_utf8fromwc(line, l, wp, wcslen(wp));
 		//fprintf(OUT,"retour GetCommandLine line=%s\n",line);fflush(OUT);
 		/* extract 1st word of line or first group delimited by " " */
 		if(*line == '"') { line++; p = strchr(line, '"'); }
@@ -73,10 +75,12 @@ char *get_prog_dir(void)
 		if(p == NULL) p = line + strlen(line);
 		l = p - line;
 		while( l > 0 && line[l - 1] != '\\' ) l--;
+	  if (l == 0) {strcpy(line, ".\\"); l = 2;}
 		if(l > 0) {
 			dir = (char *)malloc(l + 1);
 			memcpy(dir, line, l); dir[l] = 0;
 		}
+		delete[] line;
 	}
 	return dir;
 }
@@ -84,17 +88,45 @@ char *get_prog_dir(void)
 
 char **getutf8args(int *pargc)
 {
-	int argc;
+	int argc, l;
 	wchar_t **wargv = CommandLineToArgvW(GetCommandLineW(), &argc);
 	char **argv = (char **)malloc(argc * sizeof(char *));
 	for (int count = 0; count < argc; count++) {
-		argv[count] = strdup(wchartoutf8(wargv[count]));
+	    l = 4*wcslen(wargv[count])+1;
+	    argv[count] = (char *)malloc(l);
+	    fl_utf8fromwc(argv[count], l, wargv[count], wcslen(wargv[count]));
 	}
 	*pargc = argc;
 	return argv;
 }
 
 
+int mysystem(const char *cmd)
+{
+  DWORD retval;
+  BOOL status;
+  //conversion of utf-8 cmd into a WCHAR string
+  int wlen = MultiByteToWideChar(CP_UTF8, 0, cmd, -1, NULL, 0);
+  WCHAR *wcmd = new WCHAR[wlen];
+  MultiByteToWideChar(CP_UTF8, 0, cmd, -1, wcmd, wlen);  
+  PROCESS_INFORMATION pi; 
+  STARTUPINFOW si;
+  ZeroMemory( &pi, sizeof(PROCESS_INFORMATION) );
+  ZeroMemory( &si, sizeof(STARTUPINFOW) );
+  si.cb = sizeof(STARTUPINFOW);
+  si.dwFlags = STARTF_USESTDHANDLES;
+  si.hStdOutput = GetStdHandle(STD_OUTPUT_HANDLE);
+  si.hStdError = GetStdHandle(STD_ERROR_HANDLE);
+  si.hStdInput = GetStdHandle(STD_INPUT_HANDLE);
+  status = CreateProcessW(NULL, wcmd, 0,0,TRUE,CREATE_NO_WINDOW,0,0, &si, &pi);
+  if (!status) return 1;
+  do retval = WaitForSingleObject(pi.hProcess, 500);
+  while (retval == WAIT_TIMEOUT);
+  delete[] wcmd;
+  return 0;
+}
+
+
 const char *win32_UTF8_to_CP1252(const char *in)
 {
   static char dst[250];
@@ -102,7 +134,6 @@ const char *win32_UTF8_to_CP1252(const char *in)
   return (const char *)dst;
 }
 
-extern LPCWSTR utf8towchar(const char *in);
 
 static size_t pdf_write_proc(PDF *p, void *data, size_t size)
 {
@@ -123,7 +154,7 @@ PDF *win32_PDF_new()
 int win32_PDF_begin_document(PDF *p, const char *filename, int len, const char *optlist)
 {
   FILE **output = (FILE**)PDF_get_opaque(p);
-  *output = _wfopen(utf8towchar(filename), L"wb"); // binary mode compulsory
+  *output = fl_fopen(filename, "wb"); // binary mode compulsory
   if (*output == NULL) return -1;
   PDF_begin_document_callback(p, pdf_write_proc, optlist);
   return 1;
@@ -147,6 +178,35 @@ void win32_PDF_delete(PDF *p)
   PDF_delete(p);
 }
 
+/* to delete all "base_fname*" files
+ */
+void delete_tmp_filename(const char *base_fname)
+{
+  WIN32_FIND_DATAW data;
+  HANDLE h;
+  WCHAR fname[MAX_PATH];
+  unsigned l = strlen(base_fname);
+  unsigned wn = fl_utf8toUtf16(base_fname, l, NULL, 0) + 2; // Query length
+  WCHAR *wbuf = new WCHAR[wn];
+  wn = fl_utf8toUtf16(base_fname, l, (unsigned short*)wbuf, wn); // Convert string to WCHAR[]
+  wbuf[wn] = '*'; // add '*' at end of base name
+  wbuf[wn + 1] = 0;
+  WCHAR *p = wbuf + wn; // find end of directory part in base_fname
+  while (*p != '\\') p--;
+  l = p - wbuf + 1;
+  memcpy(fname, wbuf, l * sizeof(WCHAR)); // memorize directory part
+  h = FindFirstFileW(wbuf, &data);
+  delete[] wbuf;
+  if (h  != INVALID_HANDLE_VALUE) {
+    do {
+      memcpy(fname + l, data.cFileName, (wcslen(data.cFileName)+1)*sizeof(WCHAR)); // build full pathname
+      DeleteFileW(fname);
+      }
+    while (FindNextFileW(h, &data) != 0);
+    FindClose(h);
+    }
+}
+
 /*
  void add_to_path(char *dir)
  /* ajoute a la variable PATH la directory dir
